#!/usr/local/bin/perl

# Copyright (C) 2003 by Johannes Grassler. All rights reserved. This is
# free software and can be distributed under the terms of either the
# Artistic License or the GNU General Public License which can be found
# in the files Artistic and COPYING.

use strict;
use warnings;

use Getopt::Std;
use Digest::MD5 qw"md5_hex md5";

my %opts;
getopts('o:f:d:h', \%opts);

my $usage = "usage: $0 -h -o <output file> -f <field1;field2;...;fieldn> -d <field1;field2;...;fieldn> [input file]\n";

if ( $opts{h} ) { die $usage; }

my $sourcefile = "-";
if ( $ARGV[0] ) { $sourcefile = $ARGV[0]; }

my $outputfile = "-";
if ( $opts{o} ) { $outputfile = $opts{o}; }

my @fields = (1,3);
if ( $opts{f} ) { @fields = split(/;/, $opts{f}); }

# Discard these fields in output
my @fields_discard; 
if ( $opts{d} ) { @fields_discard = split(/;/, $opts{d}); }

open SOURCE,$sourcefile;
open DEST, ">$outputfile";

while ( my $line = <SOURCE> )
	{
	chomp $line;
	my @fields_in = split (/\t+/, $line);
	my $digest_base;
	foreach my $field ( @fields )
		{
		$digest_base .= $fields_in[$field];
		}

	# discard fields that are not to be output

	foreach my $field ( @fields_discard )
		{
		splice (@fields_in, $field - 1, 1);
		}

	my $digest = md5_hex ($digest_base);
	my $line_out;
	foreach my $field ( @fields_in )
		{
		$line_out .= $field . "\t";
		}
	$line_out .= "$digest\n";
	print DEST $line_out;
	}

close SOURCE;
close DEST;
