#!/usr/local/bin/perl

# Copyright (C) 2003 by Johannes Grassler. All rights reserved. This is
# free software and can be distributed under the terms of either the
# Artistic License or the GNU General Public License which can be found
# in the files Artistic and COPYING.

use strict;
use Getopt::Std;


my %opts;
getopts('o:f:u:g:G:h', \%opts);

# Set the GID for the affected users
my $target_gid = 2001;
if ( $opts{G} ) 
	{ 
	if ( $opts{G} =~ /^\d+$/ ) 
		{ 
		if ( ! getgrgid($opts{G}) ) { die "Group $opts{G} doesn't exist.\n"; }
		$target_gid = $opts{G};
		}
	else
		{ 
		if ( ! getgrnam($opts{G}) ) { die "Group $opts{G} doesn't exist.\n"; }
		$target_gid = $opts{G}; 
		}
	}

# Set the list of users who won't be marked for deletion even if their
# sums are not found in the input file

my @protected_users;
if ( $opts{u} ) { @protected_users = split(/;/, $opts{u}); }

# Set the list of groups whose members won't be marked for deletion even
# if their sums are not found in the input file
my @protected_groups;
if ( $opts{g} ) { @protected_groups = split(/;/, $opts{g}); }

my $input = "-";
if ( $ARGV[0] ) { $input = $ARGV[0]; }

my $output = "-";
if ( $opts{o} ) { $output = $opts{o}; }

my $field = 2;
if ( $opts{f} )
	{ 
	if ( $opts{f} !~ /^\d+$/ ) { die "Argument to -f is invalid. Please specify a numeric value.\n"; }
	else { $field = $opts{f} - 1; }
	}


my $usage = "usage: $0 -o <output file> -f <MD5 sum field in GECOS> -g <group1;...;group n> -u <user1;...;user n> -G <affected group> [input file]\n";
if ( $opts{h} ) { die $usage; }

foreach my $protected_group (@protected_groups)
	{
	my (undef, undef, undef, $memberlist) = getgrnam($protected_group);
	my @members = split(/\s/, $memberlist);
	push @protected_users, @members;
	}

my @remove;

# Read the list of MD5 sums to be checked against
my @sums;
open INPUT,$input or die "Couldn't open $input: $!\n";
while ( my $line = <INPUT> )
	{
	my (undef, undef, undef, $checksum) = split(/\t/, $line);
	chomp $checksum;
	push @sums, $checksum;
	}

# open the output filehandle

open OUTPUT,">$output";

# Iterate through the passwd file
PWDPARSE: while ( my ($username, undef, undef, $gid, undef, undef, $gecos) = getpwent() )
	{
	# First check if the user is in the target group and skip to the
	# next iteration if not
	if ( $gid ne $target_gid ) { next PWDPARSE; }

	# Skip protected users
	foreach my $skipuser (@protected_users)
		{
		if ( $username eq $skipuser ) { next PWDPARSE; }
		}

	# Now check the sum of the user entry against the sum from the list
	my @gecos_entries = split(/;/, $gecos);
	my $sum = $gecos_entries[$field];
	foreach my $checksum (@sums)
		{
		if ( $sum eq $checksum ) { next PWDPARSE; }
		}
	# If we arrive here write the user to the list of users to be removed
	print OUTPUT "$username\n";
	}
